/* $Workfile: Dsp55Api.c $   $Revision: 27 $    $Modtime: 10/29/09 11:40a $  */
/*--------------------------------------------------------------------------*\

  Odin TeleSystems Inc.

  SAA-1012-1 OTX DSP C55x Software Development Kit (SDK)

  Implementation of the Standard API functions that do not necessarily have
  to be modified by the user. Some modification might be needed if a different 
  peripheral configuration is desired.

\*--------------------------------------------------------------------------*/

#include <string.h>  // for memset() 
#include <gsm.h>     // ETSI functions for mapping to intrinsics    


#include "Dsp55Ioctl.h"
#include "OtxRev.h"

#include "Dsp55Apicfg.h" 
#include <csl.h>
#include <csl_dma.h>
#include <csl_mcbsp.h>
#include <csl_irq.h>
#include <csl_chip.h>
#include <csl_timer.h>
#include <csl_pll.h>


#include "Dsp55Api.h"

#include <lck.h>

/*--------------------------------------------------------------------------*\
  Macros
\*--------------------------------------------------------------------------*/



/*--------------------------------------------------------------------------*\
  Global Data
\*--------------------------------------------------------------------------*/
#include "Dsp55Law.h" // Lookup tables for a-law and u-law decompression

// OTXDSP_HWY_SPEEDS could be specified on the command line, but if it is not we default to 2M support only
#ifndef OTXDSP_HWY_SPEEDS
unsigned short g_nSupportedHighwaySpeeds = OTXDSP_HWY_SPEED_2M; // Bit pattern, 0x0001=2M support only
#else
unsigned short g_nSupportedHighwaySpeeds = OTXDSP_HWY_SPEEDS;   // Bit pattern of supported highway speeds
#endif

volatile unsigned short *g_h2dCmdAddrP      = (unsigned short *) (OTXDSP_SDK_IOCTL_CODE_ADDR/2);
volatile unsigned short *g_h2dHandleAddrP   = (unsigned short *) (OTXDSP_SDK_IOCTL_HANDLE_ADDR/2);
volatile unsigned short *g_h2dTaskRefAddrP  = (unsigned short *) (OTXDSP_SDK_IOCTL_TASKREF_ADDR/2);
volatile unsigned short *g_h2dDataLenAddrP  = (unsigned short *) (OTXDSP_SDK_IOCTL_DATA_LEN_ADDR/2);
volatile unsigned short *g_h2dDataAddrP     = (unsigned short *) (OTXDSP_SDK_IOCTL_DATA_ADDR/2);

static unsigned short *g_pCurrDspWriteAddr  = (unsigned short *) (OTXDSP_SDK_EVENT_BUF_ADDR/2);

// OTXDSP_DYNAMIC_BSP_DMABUF is defined when BSP DMA buffers should be allocated by the user
#ifndef OTXDSP_DYNAMIC_BSP_DMABUF

#if 1
#pragma DATA_SECTION(g_sqDmaRxBuf,"dmaRxMem")
unsigned short g_sqDmaRxBuf[128*OTXDSP_SDK_FRAMES_PER_BUF*2];
#pragma DATA_SECTION(g_sqDmaTxBuf,"dmaTxMem")
unsigned short g_sqDmaTxBuf[128*OTXDSP_SDK_FRAMES_PER_BUF*2];
#else
#pragma DATA_SECTION(g_sqDmaRxBuf,"DmaBuf")
unsigned short g_sqDmaRxBuf[128*OTXDSP_SDK_FRAMES_PER_BUF*2];
#pragma DATA_SECTION(g_sqDmaTxBuf,"DmaBuf")
unsigned short g_sqDmaTxBuf[128*OTXDSP_SDK_FRAMES_PER_BUF*2];
#endif

#else
unsigned short *g_sqDmaRxBuf;
unsigned short *g_sqDmaTxBuf;

#endif

LCK_Attrs  LockAttrs;
LCK_Handle hLock;

OtxDspHpiEventBufDiag g_hpiEventBufDiag;

OtxDspTimeStamp g_timeStamp;

// Set in OtxDspStartProgram()
unsigned short g_nHighwaySpeed;
unsigned short g_nTimeslotsPerFrame;
unsigned short g_nWordslotsPerFrame;

volatile short g_bGotDmaRxInterrupt = OTXDSP_FALSE;
volatile short g_bGotDmaTxInterrupt = OTXDSP_FALSE;
volatile short g_bGotHostPortInterrupt = OTXDSP_FALSE;
volatile short g_bDmaRxHalf = OTXDSP_FALSE;
volatile short g_bDmaTxHalf = OTXDSP_FALSE;

volatile unsigned short g_nFrameBufferCounter = 0;

//static volatile short g_blinkLed0Val = OTXDSP_FALSE;

Uint32 g_nRevtCounter = 0;
Uint32 g_nXevtCounter = 0;
Uint16 g_nDmaRxTimeoutCounter = 0;
Uint16 g_nDmaRxDropCounter = 0;
Uint16 g_nDmaTxTimeoutCounter = 0;
Uint16 g_nDmaTxDropCounter = 0;

unsigned short g_debugData[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
unsigned short g_debugBuf[32] = { 0, 0, 0, 0, 0, 0, 0, 0,
                                  0, 0, 0, 0, 0, 0, 0, 0,    
                                  0, 0, 0, 0, 0, 0, 0, 0,
                                  0, 0, 0, 0, 0, 0, 0, 0 };

DMA_Handle hDma0Rx;
DMA_Handle hDma1Tx;
MCBSP_Handle hMcbsp0;


MCBSP_Config mcbspCfg0= {
  MCBSP_SPCR1_RMK(
    MCBSP_SPCR1_DLB_OFF,                   // DLB    = 0 
    MCBSP_SPCR1_RJUST_RZF,                 // RJUST  = 0 
    MCBSP_SPCR1_CLKSTP_DISABLE,            // CLKSTP = 0 
    MCBSP_SPCR1_DXENA_NA,                  // DXENA  = 0 
    MCBSP_SPCR1_ABIS_DISABLE,              // ABIS   = 0 
    MCBSP_SPCR1_RINTM_RRDY,                // RINTM  = 0 
    0,                                     // RSYNCER = 0 
    0,                                     // RFULL = 0 N/A 
    0,                                     // RRDY = 0 N/A 
    MCBSP_SPCR1_RRST_DISABLE               // RRST   = 0 
   ),
    MCBSP_SPCR2_RMK(
    MCBSP_SPCR2_FREE_NO,                   // FREE   = 0 
    MCBSP_SPCR2_SOFT_NO,                   // SOFT   = 0 
    0, //MCBSP_SPCR2_FRST_DEFAULT,                  // FRST   = 0
    0, //MCBSP_SPCR2_GRST_DEFAULT,                 // GRST   = 0 
    0, //MCBSP_SPCR2_XINTM_XRDY,                // XINTM  = 0 
    0,                                     // XSYNCER = N/A 
    0,                                     // XEMPTY = N/A 
    0,                                     // XRDY   = N/A             
    MCBSP_SPCR2_XRST_DISABLE               // XRST   = 0 
   ),
  MCBSP_RCR1_RMK( 
  MCBSP_RCR1_RFRLEN1_OF(127),              // RFRLEN1 = 128 words per frame (could be modified by OtxDspStartProgram() before this structure is used)
  MCBSP_RCR1_RWDLEN1_8BIT                  // RWDLEN1 = 000b; 8 bits per word
  ),
  MCBSP_RCR2_RMK(    
    MCBSP_RCR2_RPHASE_SINGLE,              // RPHASE  = 0; Single phase 
    MCBSP_RCR2_RFRLEN2_OF(0),              // RFRLEN2 = 0; Not used 
    MCBSP_RCR2_RWDLEN2_8BIT,               // RWDLEN2 = 0; Not used 
    MCBSP_RCR2_RCOMPAND_MSB,               // RCOMPAND = 0; No companding, MSB first
    MCBSP_RCR2_RFIG_NO, //YES,             // RFIG    = 0, Restart frame on any sync pulse (set RSYNCERR if error) 
    MCBSP_RCR2_RDATDLY_0BIT                // RDATDLY = 1 
    ),  
  MCBSP_XCR1_RMK(    
    MCBSP_XCR1_XFRLEN1_OF(127),            // XFRLEN1 = 128 words per frame (could be modified by OtxDspStartProgram() before this structure is used)
    MCBSP_XCR1_XWDLEN1_8BIT                // XWDLEN1 = 000b; 8 bits per word 
  ),   
  MCBSP_XCR2_RMK(   
    MCBSP_XCR2_XPHASE_SINGLE,              // XPHASE  = 0; Single phase 
    MCBSP_XCR2_XFRLEN2_OF(0),              // XFRLEN2 = 0; Not used 
    MCBSP_XCR2_XWDLEN2_8BIT,               // XWDLEN2 = 0; Not used 
    MCBSP_XCR2_XCOMPAND_MSB,               // XCOMPAND = 0; No companding, MSB first
    MCBSP_XCR2_XFIG_NO, //YES              // XFIG    = 0, Restart frame on any sync pulse 
    MCBSP_XCR2_XDATDLY_0BIT                // XDATDLY = 1; 1 bit delay after the frame sync 
  ),            
  MCBSP_SRGR1_DEFAULT,   
  MCBSP_SRGR2_DEFAULT,  
  MCBSP_MCR1_DEFAULT,
  MCBSP_MCR2_DEFAULT, 
  MCBSP_PCR_RMK(
    MCBSP_PCR_IDLEEN_RESET,                 // IDLEEN   = 0   
    MCBSP_PCR_XIOEN_SP,                     // XIOEN    = 0   
    MCBSP_PCR_RIOEN_SP,                     // RIOEN    = 0   
    MCBSP_PCR_FSXM_EXTERNAL,                // FSXM     = 0   
    MCBSP_PCR_FSRM_EXTERNAL,                // FSRM     = 0   
    MCBSP_PCR_SCLKME_NO,                    // SCLKME   = 0   
    0,                                      // CLKSSTAT = N/A 
    0,                                      // DXSTAT = N/A   
    0,                                      // DRSTAT = N/A   
    MCBSP_PCR_CLKXM_INPUT,                  // CLKXM    = 0   
    MCBSP_PCR_CLKRM_INPUT,                  // CLKRM    = 0   
    MCBSP_PCR_FSXP_ACTIVEHIGH,              // FSXP     = 0   
    MCBSP_PCR_FSRP_ACTIVEHIGH,              // FSRP     = 0   
    MCBSP_PCR_CLKXP_RISING, // RISING,FALLING      // CLKXP    = 1   
    MCBSP_PCR_CLKRP_FALLING                 // CLKRP    = 0   
  ), 
 MCBSP_RCERA_DEFAULT, 
 MCBSP_RCERB_DEFAULT, 
 MCBSP_RCERC_DEFAULT, 
 MCBSP_RCERD_DEFAULT, 
 MCBSP_RCERE_DEFAULT, 
 MCBSP_RCERF_DEFAULT, 
 MCBSP_RCERG_DEFAULT, 
 MCBSP_RCERH_DEFAULT, 
 MCBSP_XCERA_DEFAULT,
 MCBSP_XCERB_DEFAULT,
 MCBSP_XCERC_DEFAULT,
 MCBSP_XCERD_DEFAULT,  
 MCBSP_XCERE_DEFAULT,
 MCBSP_XCERF_DEFAULT,  
 MCBSP_XCERG_DEFAULT,
 MCBSP_XCERH_DEFAULT
};     

/* Create DMA Receive Side Configuration */
DMA_Config  dmaCfg0Rx = { 
  DMA_DMACSDP_RMK(
    DMA_DMACSDP_DSTBEN_NOBURST,
    DMA_DMACSDP_DSTPACK_OFF,
    DMA_DMACSDP_DST_SARAM,
    DMA_DMACSDP_SRCBEN_NOBURST,
    DMA_DMACSDP_SRCPACK_OFF,
    DMA_DMACSDP_SRC_PERIPH,
    DMA_DMACSDP_DATATYPE_16BIT
  ),                                       /* DMACSDP  */
  DMA_DMACCR_RMK(
    DMA_DMACCR_DSTAMODE_POSTINC,
    DMA_DMACCR_SRCAMODE_CONST,
    DMA_DMACCR_ENDPROG_ON,
    DMA_DMACCR_REPEAT_ON,
    DMA_DMACCR_AUTOINIT_ON,
    DMA_DMACCR_EN_STOP, // DMA_start() will enable later
    DMA_DMACCR_PRIO_LOW,  
    DMA_DMACCR_FS_DISABLE,
    DMA_DMACCR_SYNC_REVT0
  ),                                       /* DMACCR   */
  DMA_DMACICR_RMK(
    DMA_DMACICR_BLOCKIE_OFF,
    DMA_DMACICR_LASTIE_OFF,
    DMA_DMACICR_FRAMEIE_ON,
    DMA_DMACICR_FIRSTHALFIE_ON,
    DMA_DMACICR_DROPIE_ON,
    DMA_DMACICR_TIMEOUTIE_ON
  ),                                       /* DMACICR  */
    (DMA_AdrPtr)(MCBSP_ADDR(DRR10)),       /* DMACSSAL */
    0,                                     /* DMACSSAU */
    (DMA_AdrPtr)&(g_sqDmaRxBuf[0]),        /* DMACDSAL */
    0,                                     /* DMACDSAU */
    (128*OTXDSP_SDK_FRAMES_PER_BUF*2),     /* DMACEN 128 words per frame (could be modified by OtxDspStartProgram() before this structure is used) */
    1,                                     /* DMACFN   */
    0,                                     /* DMACFI   */
    2                                      /* DMACEI   */
};

/* Create DMA Transmit Side Configuration */

DMA_Config  dmaCfg1Tx = { 
  DMA_DMACSDP_RMK(
    DMA_DMACSDP_DSTBEN_NOBURST,
    DMA_DMACSDP_DSTPACK_OFF,
    DMA_DMACSDP_DST_PERIPH,
    DMA_DMACSDP_SRCBEN_NOBURST,
    DMA_DMACSDP_SRCPACK_OFF,
    DMA_DMACSDP_SRC_SARAM,
    DMA_DMACSDP_DATATYPE_16BIT
  ),                                       /* DMACSDP  */
  DMA_DMACCR_RMK(
    DMA_DMACCR_DSTAMODE_CONST,
    DMA_DMACCR_SRCAMODE_POSTINC, // SGLINDX, // POSTINC,
    DMA_DMACCR_ENDPROG_ON,
    DMA_DMACCR_REPEAT_ON,
    DMA_DMACCR_AUTOINIT_ON,
    DMA_DMACCR_EN_STOP, //  DMA_start() will enable later
    DMA_DMACCR_PRIO_LOW,
    DMA_DMACCR_FS_DISABLE,
    DMA_DMACCR_SYNC_XEVT0
  ),                                       /* DMACCR   */
  DMA_DMACICR_RMK(
    DMA_DMACICR_BLOCKIE_OFF,
    DMA_DMACICR_LASTIE_OFF,
    DMA_DMACICR_FRAMEIE_ON,
    DMA_DMACICR_FIRSTHALFIE_ON,
    DMA_DMACICR_DROPIE_ON,
    DMA_DMACICR_TIMEOUTIE_ON
  ),                                       /* DMACICR  */
    (DMA_AdrPtr)&(g_sqDmaTxBuf[0]),        /* DMACSSAL */
    0,                                     /* DMACSSAU */
    (DMA_AdrPtr)(MCBSP_ADDR(DXR10)),       /* DMACDSAL */
    0,                                     /* DMACDSAU */
    (128*OTXDSP_SDK_FRAMES_PER_BUF*2),     /* DMACEN 128 words per frame (could be modified by OtxDspStartProgram() before this structure is used) */
    1,                                     /* DMACFN   */
    0,                                     /* DMACFI   */
    2                                      /* DMACEI   */
};

#if 0 // Only for XINT and RINT interrupts, which are not used
//Uint16 g_nRxFrameSyncErrors = 0;
//Uint16 g_nTxFrameSyncErrors = 0;
//Uint16 g_nRxRrdyErrors = 0;
//Uint16 g_nTxXrdyErrors = 0;
//Uint16 g_nRintCounter = 0;
//Uint16 g_nXintCounter = 0;
#endif


Uint16 g_nStartTimeoutCnt = 0;
 

/*--------------------------------------------------------------------------*\
  Interrupt Service Routines
\*--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------*\
  @func

    DMA Channel 0 Interrupt Service Routine

    The DMA is triggered by the REVT0 (Serial Port 0 Receive Event). When
    the DMA receiver buffer is full or half-full, this ISR is triggered. 
    This ISR sets a number of global variables which are checked by the 
    processing task. 

\*--------------------------------------------------------------------------*/
interrupt void c_dmac0(
  void
)
{
  Uint16 readVal;
  
  g_nRevtCounter++;
  g_nFrameBufferCounter++;

  readVal = DMA_RGET(DMACSR0); 

  if (readVal & 0x0008) {
    // FRAME
    //LOG_printf(&trace, "DMA RX FRAME IE\n");
    g_bDmaRxHalf = OTXDSP_FALSE;
    g_bGotDmaRxInterrupt = OTXDSP_TRUE;
    SEM_ipost(&semProcessing);
  }

  if (readVal & 0x0004) {
    // HALF
    //LOG_printf(&trace, "DMA RX HALF IE\n");
    g_bDmaRxHalf = OTXDSP_TRUE;
    g_bGotDmaRxInterrupt = OTXDSP_TRUE;
    SEM_ipost(&semProcessing);
  } 
  
  if (readVal & 0x0002) {
    // DROP
    g_nDmaRxDropCounter++;
    LOG_printf(&trace, "DMA DROP IE\n");
  } 
   
  if (readVal & 0x0001) {
    // TIMEOUT
    g_nDmaRxTimeoutCounter++; 
    LOG_printf(&trace, "DMA TIMEOUT IE\n");
  }  
  
}



/*--------------------------------------------------------------------------*\
  @func

    DMA Channel 1 Interrupt Service Routine

    The DMA is triggered by the XEVT0 (Serial Port 0 Transmit Event). When
    the DMA transmitter buffer is depleted or half-depleted, this ISR is triggered. 
    This ISR sets a number of global variables which are checked by the 
    processing task. 

\*--------------------------------------------------------------------------*/
interrupt void c_dmac1(
  void
)
{ 
  Uint16 readVal;
  
  g_nXevtCounter++;

  readVal = DMA_RGET(DMACSR1); 

  if (readVal & 0x0008) {
    // FRAME
    //LOG_printf(&trace, "DMA TX FRAME IE\n");
    g_bDmaTxHalf = OTXDSP_FALSE;
    g_bGotDmaTxInterrupt = OTXDSP_TRUE;
    SEM_ipost(&semProcessing);
  }

  if (readVal & 0x0004) {
    // HALF
    //LOG_printf(&trace, "DMA TX HALF IE\n");
    g_bDmaTxHalf = OTXDSP_TRUE;
    g_bGotDmaTxInterrupt = OTXDSP_TRUE;
    SEM_ipost(&semProcessing);
  } 
  
  if (readVal & 0x0002) {
    // DROP
    g_nDmaTxDropCounter++;
    LOG_printf(&trace, "DMA TX DROP IE\n");
  } 
   
  if (readVal & 0x0001) {
    // TIMEOUT
    g_nDmaTxTimeoutCounter++;
    LOG_printf(&trace, "DMA TX TIMEOUT IE\n");
  } 
  
}    




/*--------------------------------------------------------------------------*\
  @func

    Serial Port Receive Interrupt Service Routine

    The DSP is connected to the system highway (32 or 128 timeslots).
    
    This ISR is not used at the moment. The receive serial port is handled
    by the DMA (Channel 0).

\*--------------------------------------------------------------------------*/
interrupt void c_rint0(
  void
)
{
/*
  Uint16 readVal;

  g_nRintCounter++;

  //LOG_printf(&trace, "MCBSP0 RINT\n");

  //readVal = MCBSP_RGET(DRR10);
  readVal = MCBSP_read16(hMcbsp0); 

  readVal = MCBSP_RGET(SPCR10); 
  if (readVal & 0x0008) {
    // RSYNCERR
    g_nRxFrameSyncErrors++;
  }  
*/
}

/*--------------------------------------------------------------------------*\
  @func

    Serial Port Transmit Interrupt Service Routine
    
    This ISR is not used at the moment. The transmit serial port is handled
    by the DMA (Channel 1).
    
\*--------------------------------------------------------------------------*/
interrupt void c_xint0(
  void
)
{ 
/*
  Uint16 readVal;
  static Uint16 nVal = 0;
  
  g_nXintCounter++;   
  
  readVal = MCBSP_RGET(SPCR20); 
  if (readVal & 0x0008) {
    // XSYNCERR
    g_nTxFrameSyncErrors++;
  }   
  
  if (MCBSP_xrdy(hMcbsp0)) {
    MCBSP_write16(hMcbsp0, 0x00FF & (nVal++));
  } else {
    g_nTxXrdyErrors++;
  } 
*/
}


/*--------------------------------------------------------------------------*\
  @func

    DSP Interrupt Interrupt #0. Driven from the host.

    Not used at the moment.

\*--------------------------------------------------------------------------*/
interrupt void c_extint0(
  void
)
{ 
  SWI_post(&ClearHint_swi);
}

Void SwiClearHint(Void)
{
  // Nothing for now
}

/*--------------------------------------------------------------------------*\
  @func

    Host Port Interrupt. Activated when the host wants our attention. 

\*--------------------------------------------------------------------------*/
interrupt void c_dspint(
  void
)
{ 
  g_bGotHostPortInterrupt = OTXDSP_TRUE;
  SEM_ipost(&semProcessing);
}

/*--------------------------------------------------------------------------*\
  @func

\*--------------------------------------------------------------------------*/
static void InitSerialPorts(
  void
)
{
  Uint16 cha0EventId;
  Uint16 cha1EventId;
  Uint16 srcAddrHi, srcAddrLo;
  Uint16 dstAddrHi, dstAddrLo;
  Uint16 FoundStart;
  Uint16 FoundStartTimeoutCnt = 0;


  // By default, the TMS320C55xx compiler assigns all data symbols word 
  // addresses. The DMA however, expects all addresses to be byte       
  // addresses. Therefore, we must shift the address by 2 in order to   
  // change the word address to a byte address for the DMA transfer.     
  srcAddrHi = (Uint16)(((Uint32)(dmaCfg0Rx.dmacssal)) >> 15) & 0xFFFFu;
  srcAddrLo = (Uint16)(((Uint32)(dmaCfg0Rx.dmacssal)) << 1) & 0xFFFFu;
  dstAddrHi = (Uint16)(((Uint32)(dmaCfg0Rx.dmacdsal)) >> 15) & 0xFFFFu;
  dstAddrLo = (Uint16)(((Uint32)(dmaCfg0Rx.dmacdsal)) << 1) & 0xFFFFu;
  
  dmaCfg0Rx.dmacssal = (DMA_AdrPtr)srcAddrLo;
  dmaCfg0Rx.dmacssau = srcAddrHi;
  dmaCfg0Rx.dmacdsal = (DMA_AdrPtr)dstAddrLo;
  dmaCfg0Rx.dmacdsau = dstAddrHi;
  
  srcAddrHi = (Uint16)(((Uint32)(dmaCfg1Tx.dmacssal)) >> 15) & 0xFFFFu;
  srcAddrLo = (Uint16)(((Uint32)(dmaCfg1Tx.dmacssal)) << 1) & 0xFFFFu;
  dstAddrHi = (Uint16)(((Uint32)(dmaCfg1Tx.dmacdsal)) >> 15) & 0xFFFFu;
  dstAddrLo = (Uint16)(((Uint32)(dmaCfg1Tx.dmacdsal)) << 1) & 0xFFFFu;
  
  dmaCfg1Tx.dmacssal = (DMA_AdrPtr)srcAddrLo;
  dmaCfg1Tx.dmacssau = srcAddrHi;
  dmaCfg1Tx.dmacdsal = (DMA_AdrPtr)dstAddrLo;
  dmaCfg1Tx.dmacdsau = dstAddrHi;
  
  // Open peripheral devices  
  hDma0Rx = DMA_open(DMA_CHA0, DMA_OPEN_RESET);
  hDma1Tx = DMA_open(DMA_CHA1, DMA_OPEN_RESET);
  hMcbsp0 = MCBSP_open(MCBSP_PORT0, MCBSP_OPEN_RESET);
  
  
  // Temporarily disable all maskable interrupts 
  // Interrupts are disabled after a hardware reset, but disable them anyway, just in case
  IRQ_globalDisable();

  // Look for FRAME Start
  MCBSP_config(hMcbsp0, &mcbspCfg0);
  
  // Enable the serial port
  MCBSP_start(hMcbsp0, MCBSP_XMIT_START | MCBSP_RCV_START, 0); 

  FoundStart = MCBSP_rrdy(hMcbsp0);
  FoundStartTimeoutCnt = 0;
  while(FoundStart == 0) {
    // Wait here until ready or timeout
    FoundStart = MCBSP_rrdy(hMcbsp0);
    FoundStartTimeoutCnt++;
    if(FoundStartTimeoutCnt == 0) {
      // Count value rolled over it is taking
      // too long to find the ready signal.
      g_nStartTimeoutCnt++;
      FoundStart = 1; // Exit loop; check frame sync
    }
  } 

  // Ready for more data McBSP must have received a frame sync
  MCBSP_reset(hMcbsp0);
  MCBSP_config(hMcbsp0, &mcbspCfg0);

  // reset error counters
  g_nDmaRxTimeoutCounter = 0;
  g_nDmaRxDropCounter    = 0;
  g_nDmaTxTimeoutCounter = 0;
  g_nDmaTxDropCounter    = 0;

  // Ready to enable McBSP and DMA

  // Configure peripheral devices    
  DMA_config(hDma0Rx, &dmaCfg0Rx); 
  DMA_config(hDma1Tx, &dmaCfg1Tx); 
  MCBSP_config(hMcbsp0, &mcbspCfg0);

  // Get Event ID's associated with DMA channels
  cha0EventId = DMA_getEventId(hDma0Rx);   
  cha1EventId = DMA_getEventId(hDma1Tx);   
  
  // Clear any pending interrupts for DMA channels
  IRQ_clear(cha0EventId);
  IRQ_clear(cha1EventId);
  //IRQ_clear(IRQ_EVT_RINT0);
  //IRQ_clear(IRQ_EVT_XINT0);
  
  IRQ_enable(cha0EventId);     // Allow DMA0 interrupts
  IRQ_enable(cha1EventId);     // Allow DMA1 interrupts
  //IRQ_enable(IRQ_EVT_RINT0);
  //IRQ_enable(IRQ_EVT_XINT0);
  
  // Place DMA interrupt service addresses at associate vector
  IRQ_plug(cha0EventId, &c_dmac0);
  IRQ_plug(cha1EventId, &c_dmac1);
  //IRQ_plug(IRQ_EVT_RINT0, &c_rint0);
  //IRQ_plug(IRQ_EVT_XINT0, &c_xint0);
  
  // Enable all maskable interrupts, INTM = 0 
  IRQ_globalEnable();
  
  // Enable DMA
  DMA_start(hDma0Rx); 
  DMA_start(hDma1Tx); 


  // Enable the serial port
  MCBSP_start(hMcbsp0, MCBSP_XMIT_START | MCBSP_RCV_START, 0); 

}

/*--------------------------------------------------------------------------*\
  API Functions
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
  @func

    Run Standard OTX DSP SDK API Initializations

    Note that CSL_init() is already called before main() is called.

\*--------------------------------------------------------------------------*/
void OtxDspInit(
  void
)
{          
  //Uint16 readVal;
  Uint16 nRevId;
  Uint32 nDieIdLow;
  
  nDieIdLow = CHIP_getDieId_Low32();     
  LOG_printf(&trace, "DieIdLow=0x%x\n", nDieIdLow);
                
  nRevId = CHIP_getRevId();  
  LOG_printf(&trace, "RevId=0x%x\n", nRevId);
#ifdef _DEBUG
  switch (nRevId) {
  case 0x8050:
    LOG_printf(&trace, "Silicon Version 1.x (0x%x)\n", nRevId);
    break;
  case 0x6510:
    LOG_printf(&trace, "Silicon Version 2.0 (0x%x)\n", nRevId);
    break;
  case 0x6511:
    LOG_printf(&trace, "Silicon Version 2.1 (0x%x)\n", nRevId);
    break;
  case 0x6512:
    LOG_printf(&trace, "Silicon Version 2.2 (0x%x)\n", nRevId);
    break;
  default:
    LOG_printf(&trace, "Silicon Version <unknown> (0x%x)\n", nRevId);
    break;
  }   
#endif

  // Initialize Standard Global Data
  OtxDspInitGlobalData();   

  // Temporarily disable all maskable interrupts 
  // Interrupts are disabled after a hardware reset, but disable them anyway, just in case
  IRQ_globalDisable();
  
  // Clear any pending interrupts for DMA channels
  IRQ_clear(IRQ_EVT_DSPINT);
  //IRQ_clear(IRQ_EVT_INT0);
  
  IRQ_enable(IRQ_EVT_DSPINT);  // Allow Host Port Interrupt 
  //IRQ_enable(IRQ_EVT_INT0);    // Allow INT0 interrupts (not really used at the moment) 
  
  // Place DMA interrupt service addresses at associate vector
  IRQ_plug(IRQ_EVT_DSPINT, &c_dspint);
  //IRQ_plug(IRQ_EVT_INT0, &c_extint0);
  
  // Enable all maskable interrupts, INTM = 0 
  IRQ_globalEnable();

//  C55X_SET_LED0_OFF; // Turn off LED0
//  C55X_SET_LED1_OFF; // Turn off LED1
//  C55X_SET_LED2_ON;  // The Blink LED. Indicate that the processor is initialized. 
                     // It should then start to blink when the Idle task starts
                     // running. If it does not start to blink it is an indication
                     // that some task is hogging the CPU.
 hLock = LCK_create(&LockAttrs);
  
}

/*--------------------------------------------------------------------------*\
  @func
    Initializes the Multi-channel Buffered Serial Port (MCBSP#0). This serial
    port is connected to a PCM highway on the OTX NIC board. The PCM highway
    can run at 2.048, 4.096, or 8.192 Mbps speed. 
\*--------------------------------------------------------------------------*/
void OtxDspInitSerialPort(
  void
)
{ 
  DMA_stop(hDma0Rx); 
  DMA_stop(hDma1Tx);       
  
  MCBSP_reset(hMcbsp0);
  MCBSP_config(hMcbsp0, &mcbspCfg0);
  
  // Enable DMA
  DMA_start(hDma0Rx); 
  DMA_start(hDma1Tx); 

  MCBSP_start(hMcbsp0, MCBSP_XMIT_START | MCBSP_RCV_START, 0); 
  
  //MCBSP_write16(hMcbsp0, g_sqDmaTxBuf[0]);
  
  
  // Prime MCBSP DXR
  //while (!(MCBSP_xrdy(hMcbsp0))){
  //  ;
  //}
  
}



void OtxDspInitGlobalData(
/*--------------------------------------------------------------------------*\
  @func
    Initalizes the global data used by the Standard API functions.
\*--------------------------------------------------------------------------*/
  void
)
{
  short i;
  
  g_pCurrDspWriteAddr  = (Uint16 *) (OTXDSP_SDK_EVENT_BUF_ADDR/2);

  g_hpiEventBufDiag.cwFreeBuffer = g_hpiEventBufDiag.cwMinFreeBuffer = OTXDSP_SDK_EVENT_BUF_LEN;
  g_hpiEventBufDiag.coLostEvents = 0;  
  
  memset(&g_timeStamp, 0, sizeof(OtxDspTimeStamp));
  
  // Initialize the receive buffer
  for (i = 0; i < 128*OTXDSP_SDK_FRAMES_PER_BUF*2; i++) {
    g_sqDmaRxBuf[i] = 0x1234;
  }

  // Initialize the transmit buffer
  for (i = 0; i < 128*OTXDSP_SDK_FRAMES_PER_BUF*2; i++) {
    g_sqDmaTxBuf[i] = 0; // i % 32;
  }
  
}




/*--------------------------------------------------------------------------*\
  @func
\*--------------------------------------------------------------------------*/
short OtxDspStartProgram(
  OtxDspSdkIoInStartProgramS *pSin
)
{
#ifndef CHIP_5510
  PLL_Config pllCfg;

  pllCfg.iai     = 0;  // Initalize after Idle bit [0=The PLL locks using the same process that was underway before idle mode was entered].
  pllCfg.iob     = 1;  // Initalize on break bit. [1=The clock generator switches to its bypass mode and restarts the PLL phase-locking sequence]
  pllCfg.div     = (pSin->nClkMdVal >> 5) & 0x0003;
  pllCfg.pllmult = (pSin->nClkMdVal >> 7) & 0x001F;

  PLL_config(&pllCfg);
#endif

  // Check that the driver revision matches the revision of the DSP SDK
  if (pSin->nDrvRevNoMajor != OTX_DRV_REV_NO_MAJOR) {
    return OTXDSP_SDK_E_UNSUPPORTED_VERSION;
  }
  if (pSin->nDrvRevNoMinor != OTX_DRV_REV_NO_MINOR) {
    return OTXDSP_SDK_E_UNSUPPORTED_VERSION;
  }
  if (pSin->nDrvRevNoPr != OTX_DRV_REV_NO_PR) {
    return OTXDSP_SDK_E_UNSUPPORTED_VERSION;
  }
  if (pSin->nDrvRevNoPr == 0) {
    if (pSin->nDrvRevNoPversion != OTX_DRV_REV_NO_PVERSION) {
      return OTXDSP_SDK_E_UNSUPPORTED_VERSION;
    }
  }

  g_nHighwaySpeed      = pSin->nHighwaySpeed;
  g_nTimeslotsPerFrame = g_nHighwaySpeed >> 6;
  g_nWordslotsPerFrame = g_nHighwaySpeed >> 7;
#if 1 // temp
  g_debugData[4] = g_nHighwaySpeed;
  g_debugData[5] = g_nTimeslotsPerFrame;
  g_debugData[6] = g_nWordslotsPerFrame;
#endif

  if (pSin->nHighwaySpeed == 4096) {
    if (!(g_nSupportedHighwaySpeeds & OTXDSP_HWY_SPEED_4M)) {
      return OTXDSP_SDK_E_NOT_SUPPORTED;
    }
    mcbspCfg0.rcr1 = 
      MCBSP_RCR1_RMK( 
      MCBSP_RCR1_RFRLEN1_OF(63),              // RFRLEN1 = 64 words per frame 
      MCBSP_RCR1_RWDLEN1_8BIT                 // RWDLEN1 = 000b; 8 bits per word
      );
    mcbspCfg0.xcr1 = 
      MCBSP_XCR1_RMK(    
      MCBSP_XCR1_XFRLEN1_OF(63),              // XFRLEN1 = 64 words per frame     
      MCBSP_XCR1_XWDLEN1_8BIT                 // XWDLEN1 = 000b; 8 bits per word  
      );   
    dmaCfg0Rx.dmacen = (Uint16)(64*OTXDSP_SDK_FRAMES_PER_BUF*2);
    dmaCfg1Tx.dmacen = (Uint16)(64*OTXDSP_SDK_FRAMES_PER_BUF*2);

  } else if (pSin->nHighwaySpeed == 8192) {
    if (!(g_nSupportedHighwaySpeeds & OTXDSP_HWY_SPEED_8M)) {
      return OTXDSP_SDK_E_NOT_SUPPORTED;
    }
    mcbspCfg0.rcr1 = 
      MCBSP_RCR1_RMK( 
      MCBSP_RCR1_RFRLEN1_OF(127),             // RFRLEN1 = 128 words per frame 
      MCBSP_RCR1_RWDLEN1_8BIT                 // RWDLEN1 = 000b; 8 bits per word
      );
    mcbspCfg0.xcr1 = 
      MCBSP_XCR1_RMK(    
      MCBSP_XCR1_XFRLEN1_OF(127),             // XFRLEN1 = 128 words per frame     
      MCBSP_XCR1_XWDLEN1_8BIT                 // XWDLEN1 = 000b; 8 bits per word  
      );   
    dmaCfg0Rx.dmacen = (Uint16)(128*OTXDSP_SDK_FRAMES_PER_BUF*2);
    dmaCfg1Tx.dmacen = (Uint16)(128*OTXDSP_SDK_FRAMES_PER_BUF*2);

  } else if (pSin->nHighwaySpeed == 2048) {
    // Default
    mcbspCfg0.rcr1 = 
      MCBSP_RCR1_RMK( 
      MCBSP_RCR1_RFRLEN1_OF(31),              // RFRLEN1 = 32 words per frame 
      MCBSP_RCR1_RWDLEN1_8BIT                 // RWDLEN1 = 000b; 8 bits per word
      );
    mcbspCfg0.xcr1 = 
      MCBSP_XCR1_RMK(    
      MCBSP_XCR1_XFRLEN1_OF(31),              // XFRLEN1 = 32 words per frame     
      MCBSP_XCR1_XWDLEN1_8BIT                 // XWDLEN1 = 000b; 8 bits per word  
      );   
    dmaCfg0Rx.dmacen = (Uint16)(32*OTXDSP_SDK_FRAMES_PER_BUF*2);
    dmaCfg1Tx.dmacen = (Uint16)(32*OTXDSP_SDK_FRAMES_PER_BUF*2);

  } else {
    return OTXDSP_SDK_E_INVALIDARG;
  }

  // Initialize the Serial Ports
  InitSerialPorts();

  g_debugData[0] = dmaCfg0Rx.dmacen;
  g_debugData[1] = dmaCfg1Tx.dmacen;

  return OTXDSP_SDK_S_OK;

}

/*--------------------------------------------------------------------------*\
  @func
\*--------------------------------------------------------------------------*/
short OtxDspInitTimer(
  void
)
{
  // Not implemented yet
  
  return OTXDSP_SDK_S_OK;

}



/*--------------------------------------------------------------------------*\
  @func
    Completes an IO Control call from the host. Passes a 
    arbitrary size of return data from a IO Control call back to the host.
\*--------------------------------------------------------------------------*/
void OtxDspCommandCompleteData(
  unsigned short hHandle, 
  unsigned short *pData,
  unsigned short dataLen
) 
{ 
  unsigned short i;
  unsigned short *pTempDataAddr = (unsigned short *)g_h2dDataAddrP;

  *g_h2dHandleAddrP  = hHandle;
  *g_h2dDataLenAddrP = dataLen;
  for (i = 0; i < dataLen; i++) {
    *(pTempDataAddr) = *pData++; 
    pTempDataAddr++;
  }
  *g_h2dCmdAddrP = OTXDSP_SDK_IO_NO_COMMAND; // Command completed 

  // Assert HINT# pin towards the host (active low) to interrupt the host 
  // The host will confirm the interrupt by writing a non-zero value to the interrupt confirmation address
  C55X_CLR_HINT;  // Assert the HINT# pin. I.e. interrupt the host
  C55X_NOP;       // We need to extend the pulse slightly. One NOP results in an 8ns pulse, which is suffient for the host to latch the interrupt
  C55X_NOP;       // One extra to extend the margin. This gives us an 12ns pulse.
  C55X_SET_HINT;  // Bring the HINT# high again (deasserted)
}

/*--------------------------------------------------------------------------*\
    Completes an IO Control call from the host. Passes a 
    return code from a IO Control call back to the host.
\*--------------------------------------------------------------------------*/
void OtxDspCommandCompleteRc(
  unsigned short hHandle,
  unsigned short nResult
)
{
  *g_h2dHandleAddrP  = hHandle;
  *g_h2dDataLenAddrP = sizeof(unsigned short);
  *g_h2dDataAddrP = nResult;
  *g_h2dCmdAddrP = OTXDSP_SDK_IO_NO_COMMAND; // Command completed

  // Assert HINT# pin towards the host (active low) to interrupt the host 
  // The host will confirm the interrupt by writing a non-zero value to the interrupt confirmation address
  C55X_CLR_HINT;  // Assert the HINT# pin. I.e. interrupt the host
  C55X_NOP;       // We need to extend the pulse slightly. One NOP results in an 8ns pulse, which is suffient for the host to latch the interrupt
  C55X_NOP;       // One extra to extend the margin. This gives us an 12ns pulse.
  C55X_SET_HINT;  // Bring the HINT# high again (deasserted)
}


/*--------------------------------------------------------------------------*\
    Completes an IO Control call from the host. No data is passed back
    to the host.
\*--------------------------------------------------------------------------*/
void OtxDspCommandComplete(
  unsigned short hHandle
)
{
  *g_h2dHandleAddrP  = hHandle;
  *g_h2dDataLenAddrP = 0;
  *g_h2dCmdAddrP = OTXDSP_SDK_IO_NO_COMMAND; // Command completed 

  // Assert HINT# pin towards the host (active low) to interrupt the host 
  // The host will confirm the interrupt by writing a non-zero value to the interrupt confirmation address
  C55X_CLR_HINT;  // Assert the HINT# pin. I.e. interrupt the host
  C55X_NOP;       // We need to extend the pulse slightly. One NOP results in an 8ns pulse, which is suffient for the host to latch the interrupt
  C55X_NOP;       // One extra to extend the margin. This gives us an 12ns pulse.
  C55X_SET_HINT;  // Bring the HINT# high again (deasserted)
}


/*--------------------------------------------------------------------------*\
  @func
    Easy access function for sending an event to the host with unsigned
    long as the data.
\*--------------------------------------------------------------------------*/
short OtxDspSetHostEventUlong(
  unsigned short nEvent,
  unsigned short hHandle,
  unsigned long data
)
{
  return OtxDspSetHostEventEx(nEvent, hHandle, OTXDSP_NO_TASKREF, (unsigned short *)&data, 2*sizeof(unsigned short), OTXDSP_FALSE);
}

/*--------------------------------------------------------------------------*\
  @func
    Easy access function for sending an event to the host with an unsigned
    short as the data.
\*--------------------------------------------------------------------------*/
short OtxDspSetHostEventUshort(
  unsigned short nEvent,
  unsigned short hHandle,
  unsigned short data
)
{
  return OtxDspSetHostEventEx(nEvent, hHandle, OTXDSP_NO_TASKREF, (unsigned short *)&data, sizeof(unsigned short), OTXDSP_FALSE);
}

/*--------------------------------------------------------------------------*\
  @func
    Easy access function for sending an event to the host with no data.
\*--------------------------------------------------------------------------*/
short OtxDspSetHostEventVoid(
  unsigned short nEvent,
  unsigned short hHandle
)
{
  return OtxDspSetHostEventEx(nEvent, hHandle, OTXDSP_NO_TASKREF, 0, 0, OTXDSP_FALSE);
}



/*--------------------------------------------------------------------------*\
  @func
    Generates a driver event for the host
\*--------------------------------------------------------------------------*/
short OtxDspSetHostEvent(
  unsigned short nEvent,         /* @parm Event to be send to the host */
  unsigned short hHandle,        /* @parm Handle to the user logical device sending the event */
  unsigned short nTaskRef,       /* @parm Associated Task Reference */
  unsigned short *pData,         /* @parm Pointer to the data to be sent to the host */
  unsigned short nDataLen        /* @parm Length of the data to be sent */
)
{
  return OtxDspSetHostEventEx(
    nEvent,         
    hHandle,        
    nTaskRef,       
    pData,         
    nDataLen,
    OTXDSP_FALSE   // nData is specified in words (not octets)
    );
}


/*--------------------------------------------------------------------------*\
  @func
\*--------------------------------------------------------------------------*/
inline void CalculateFreeBufferSpace(void)
{
  volatile unsigned short nCurrHostReadAddr = (*((unsigned short *) (OTXDSP_SDK_EVENT_BUF_PROCESSING_ADDR/2))) >> 1;

  // Note that the event buffer is located in an address range < 0x10000. Therefore
  // we use 'unsigned short' as the type for address pointer (to increase effiency).
  // However, the compiler provides a warning for the following statement:
  //   (unsigned short)g_pCurrDspWriteAddr 
  // Therefore we use the following cast instead (to eliminate the compiler warning)
  //   (unsigned long)g_pCurrDspWriteAddr 
  //
  if ((unsigned long)g_pCurrDspWriteAddr >= nCurrHostReadAddr) {
    g_hpiEventBufDiag.cwFreeBuffer = OTXDSP_SDK_EVENT_BUF_LEN - ((unsigned long)g_pCurrDspWriteAddr - nCurrHostReadAddr);
  } else {
    g_hpiEventBufDiag.cwFreeBuffer = ((unsigned short)nCurrHostReadAddr - (unsigned long)g_pCurrDspWriteAddr);
  }

  if (g_hpiEventBufDiag.cwFreeBuffer < g_hpiEventBufDiag.cwMinFreeBuffer) {
    g_hpiEventBufDiag.cwMinFreeBuffer = g_hpiEventBufDiag.cwFreeBuffer;
  }
}

/*--------------------------------------------------------------------------*\
  @func
\*--------------------------------------------------------------------------*/
void OtxDspCalculateFreeBufferSpace(void)
{
  CalculateFreeBufferSpace();
}


/*--------------------------------------------------------------------------*\
    Generates a driver event for the host
\*--------------------------------------------------------------------------*/
short OtxDspSetHostEventEx(
  unsigned short nEvent,         /* @parm Event to be send to the host */
  unsigned short hHandle,        /* @parm Handle to the user logical device sending the event */
  unsigned short nTaskRef,       /* @parm Associated Task Reference */
  unsigned short *pData,         /* @parm Pointer to the data to be sent to the host */
  unsigned short nDataLen,       /* @parm Length of the data to be sent */
  short bOctetTransfer           /* @parm OTXDSP_TRUE if nDataLen and pData specifies octets. OTXDSP_FALSE if they specify words */
)
{
  unsigned short i;
  volatile unsigned short *pEventAddr;

 //LCK_pend(hLock, SYS_FOREVER); // Lock

  // Use LED0 as the activity LED for the Notification Events
//  C55X_SET_LED0_ON;    
  
  CalculateFreeBufferSpace();

  // If we have enough free buffer space then insert the command in the buffer
  if (g_hpiEventBufDiag.cwFreeBuffer >= (nDataLen + 16)) {   // +16 for the nEvent+handle+taskref+timestamp*4+NO_CMD
    // Don't write the event notify code yet. 
    // Just save the address where we want to write the code. 
    // We will write the event notify code when we have written the entire event packet 
    // to the buffer
    pEventAddr = g_pCurrDspWriteAddr; 

    g_pCurrDspWriteAddr++;
    if (g_pCurrDspWriteAddr >= (unsigned short *)(OTXDSP_SDK_EVENT_BUF_ADDR/2 + OTXDSP_SDK_EVENT_BUF_LEN)) {
      g_pCurrDspWriteAddr = (unsigned short *)(OTXDSP_SDK_EVENT_BUF_ADDR/2);
    }

#ifdef OTXDSP_SDK_EVENT_BUF_INCLUDE_HANDLE
    *g_pCurrDspWriteAddr = hHandle;

    g_pCurrDspWriteAddr++;
    if (g_pCurrDspWriteAddr >= (unsigned short *)(OTXDSP_SDK_EVENT_BUF_ADDR/2 + OTXDSP_SDK_EVENT_BUF_LEN)) {
      g_pCurrDspWriteAddr = (unsigned short *)(OTXDSP_SDK_EVENT_BUF_ADDR/2);
    }
#endif

#ifdef OTXDSP_SDK_EVENT_BUF_INCLUDE_TASKREF
    *g_pCurrDspWriteAddr = nTaskRef;

    g_pCurrDspWriteAddr++;
    if (g_pCurrDspWriteAddr >= (unsigned short *)(OTXDSP_SDK_EVENT_BUF_ADDR/2 + OTXDSP_SDK_EVENT_BUF_LEN)) {
      g_pCurrDspWriteAddr = (unsigned short *)(OTXDSP_SDK_EVENT_BUF_ADDR/2);
    }
#endif

#ifdef OTXDSP_SDK_EVENT_BUF_INCLUDE_TIMESTAMP
    // Time stamp
    { 
      int old_intm;
      
      old_intm = IRQ_globalDisable(); // Globally disable interrupts by setting the INTM bit 

      *g_pCurrDspWriteAddr = g_timeStamp.nHighPart;
      
      g_pCurrDspWriteAddr++;
      if (g_pCurrDspWriteAddr >= (unsigned short *)(OTXDSP_SDK_EVENT_BUF_ADDR/2 + OTXDSP_SDK_EVENT_BUF_LEN)) {
        g_pCurrDspWriteAddr = (unsigned short *)(OTXDSP_SDK_EVENT_BUF_ADDR/2);
      }

      *g_pCurrDspWriteAddr = g_timeStamp.nMidPart;
      
      g_pCurrDspWriteAddr++;
      if (g_pCurrDspWriteAddr >= (unsigned short *)(OTXDSP_SDK_EVENT_BUF_ADDR/2 + OTXDSP_SDK_EVENT_BUF_LEN)) {
        g_pCurrDspWriteAddr = (unsigned short *)(OTXDSP_SDK_EVENT_BUF_ADDR/2);
      }

      *g_pCurrDspWriteAddr = g_timeStamp.nLowPart;
      
      g_pCurrDspWriteAddr++;
      if (g_pCurrDspWriteAddr >= (unsigned short *)(OTXDSP_SDK_EVENT_BUF_ADDR/2 + OTXDSP_SDK_EVENT_BUF_LEN)) {
        g_pCurrDspWriteAddr = (unsigned short *)(OTXDSP_SDK_EVENT_BUF_ADDR/2);
      }

      *g_pCurrDspWriteAddr = 0xFFFF - TIMER_RGET(TIM);
      
      g_pCurrDspWriteAddr++;
      if (g_pCurrDspWriteAddr >= (unsigned short *)(OTXDSP_SDK_EVENT_BUF_ADDR/2 + OTXDSP_SDK_EVENT_BUF_LEN)) {
        g_pCurrDspWriteAddr = (unsigned short *)(OTXDSP_SDK_EVENT_BUF_ADDR/2);
      }

      IRQ_globalRestore(old_intm);;  // Globally enable interrupts by clearing the INTM bit
    }
#endif

    if (bOctetTransfer) {
      *(g_pCurrDspWriteAddr) = (nDataLen + 1) / 2; // data length
    } else {
      *(g_pCurrDspWriteAddr) = nDataLen; // data length
    }

    g_pCurrDspWriteAddr++;
    if (g_pCurrDspWriteAddr >= (unsigned short *)(OTXDSP_SDK_EVENT_BUF_ADDR/2 + OTXDSP_SDK_EVENT_BUF_LEN)) {
      g_pCurrDspWriteAddr = (unsigned short *)(OTXDSP_SDK_EVENT_BUF_ADDR/2);
    }

    if (bOctetTransfer) {
      // We will only transfer the LSB of the word buffer passed to this function
      for (i = 0; i < nDataLen/2; i++) {
        *(g_pCurrDspWriteAddr)  = *pData++ & 0x00FF; 
        *(g_pCurrDspWriteAddr) |= *pData++ << 8 ; 

        g_pCurrDspWriteAddr++;
        if (g_pCurrDspWriteAddr >= (unsigned short *)(OTXDSP_SDK_EVENT_BUF_ADDR/2 + OTXDSP_SDK_EVENT_BUF_LEN)) {
          g_pCurrDspWriteAddr = (unsigned short *)(OTXDSP_SDK_EVENT_BUF_ADDR/2);
        }
      }

      if (nDataLen & 0x0001) {
        // Transferring an odd number of octets. We are padding with an extra 0x00 octet
        nEvent |= OTXDSP_SDK_MASK_EVENT_DATA_PADDED;

        *(g_pCurrDspWriteAddr)  = *pData++ & 0x00FF; 

        g_pCurrDspWriteAddr++;
        if (g_pCurrDspWriteAddr >= (unsigned short *)(OTXDSP_SDK_EVENT_BUF_ADDR/2 + OTXDSP_SDK_EVENT_BUF_LEN)) {
          g_pCurrDspWriteAddr = (unsigned short *)(OTXDSP_SDK_EVENT_BUF_ADDR/2);
        }

      } 

    } else {
      for (i = 0; i < nDataLen; i++) {
        *(g_pCurrDspWriteAddr) = *pData++; 
        
        g_pCurrDspWriteAddr++;
        if (g_pCurrDspWriteAddr >= (unsigned short *)(OTXDSP_SDK_EVENT_BUF_ADDR/2 + OTXDSP_SDK_EVENT_BUF_LEN)) {
          g_pCurrDspWriteAddr = (unsigned short *)(OTXDSP_SDK_EVENT_BUF_ADDR/2);
        }
      }
    }

    *(g_pCurrDspWriteAddr) = OTXDSP_SDK_EVENT_NO_COMMAND; // We put this here so that host knows when to stop reading from the command buffer

    // Now it is safe to write the event notify code (now when the entire packet is written to the buffer)
    *(pEventAddr) = nEvent; 

  } else {
    // Buffer overrun. The host is too slow. Record this error condition
    g_hpiEventBufDiag.coLostEvents++;
    return 0; // Error
  }

  // Assert HINT# pin towards the host (active low) to interrupt the host 
  // The host will confirm the interrupt by writing a non-zero value to the interrupt confirmation address
  C55X_CLR_HINT;  // Assert the HINT# pin. I.e. interrupt the host
  C55X_NOP;       // We need to extend the pulse slightly. One NOP results in an 8ns pulse, which is suffient for the host to latch the interrupt
  C55X_NOP;       // One extra to extend the margin. This gives us an 12ns pulse.
  C55X_SET_HINT;  // Bring the HINT# high again (deasserted)

//  C55X_SET_LED0_OFF;   

//  LCK_post(hLock); // Unlock

  return 1; // Success

}


/*--------------------------------------------------------------------------*\
  @func
    Expands an 8-bit PCM value (coded either in a-law or u-law) to a 14-bit
    sign magnitude value. 
\*--------------------------------------------------------------------------*/
short OtxDspLawExpand(
  unsigned short lawExpandInput,    /* @parm Sample to be expanded (8-bits sign magnitude value */
  short law                /* @parm Law (0 = u-law, 1 = A-law) */
)
{
  if (law == OTXDSP_ULAW) {
    return g_sqUlawExpansionTable[lawExpandInput & 0x00FF];
  } else if (law == OTXDSP_ALAW) {
    return g_sqAlawExpansionTable[lawExpandInput & 0x00FF];
  } else {
    // Error. Unsupported law
    return 0;
  }
}          


/*--------------------------------------------------------------------------*\
  @func
    Compresses a 14-bit sign magnitude value to an 8-bit u-law PCM value.
\*--------------------------------------------------------------------------*/
short OtxDspUlawCompress(
  short lawCompInput
)
{ 
  unsigned short temp;
  unsigned short nLeadingZeros; 
  unsigned short chord;
  
  temp = abs_s(lawCompInput) + 33;  // Work with positive # only and Add ulaw bias value (33)
        
  nLeadingZeros = norm_s(temp);   // Calculate leading zeros
  
  chord = 10 - nLeadingZeros;
  
  temp = (temp >> chord) & 0x000F;  // Step should now be in the 4 LSB bits
  
  chord -= 1;
    
  temp |= chord << 4; // Stuff in the chord
  
  if (lawCompInput < 0) {  // Restore the polarity bit
    temp |= 0x80;
  } 
  
  temp ^= 0xFF;   // Invert PCM result
  
  return temp;                                
}
            
/*--------------------------------------------------------------------------*\
  @func
    Compresses a 14-bit sign magnitude value to an 8-bit a-law PCM value.
\*--------------------------------------------------------------------------*/
short OtxDspAlawCompress(
  short lawCompInput
) 
{
  unsigned short temp;
  unsigned short nLeadingZeros; 
  unsigned short chord;
  
  temp = abs_s(lawCompInput);  // Work with positive numbers
  
  temp >>= 1;                  // We work with 14-bit sign magnitude value in our algorithms. A-law is 13-bit sign magnitude. So divide by 2.
  
  if (temp & 0xFFE0) { 
    // Input value (abs) is >= 32
    nLeadingZeros = norm_s(temp);   // Calculate leading zeros
    
    chord = 10 - nLeadingZeros;
    
    temp = (temp >> chord) & 0x000F;  // Step should now be in the 4 LSB bits
          
    temp |= chord << 4; // Stuff in the chord
  
  } else {
    // Input value (abs) is < 32
    temp >>= 1;   // Cord 0. We are done.
  }
  
  if (lawCompInput < 0) {  // Restore the polarity bit
    temp |= 0x80;
  } 
  
  temp ^= 0x55;   // Invert every other bit. We have our PCM result    
  
  return temp;
  
}
            

